// cl-translate-file.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2002      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

void lptrans(Object* statement) {
  // Translate `statement' to Common-Lisp and print the result.
  helpLptrans(statement);
}

void ptrans(Object* statement) {
  // Translate `statement' to Common-Lisp and print the result.
  helpLptrans(statement);
}

void helpLptrans(Object* statement) {
  { 
    BIND_STELLA_SPECIAL(oTRANSLATOROUTPUTLANGUAGEo, Keyword*, oTRANSLATOROUTPUTLANGUAGEo.get());
    BIND_STELLA_SPECIAL(oPRINTPRETTYCODEpo, boolean, TRUE);
    setTranslatorOutputLanguage(KWD_CL_TRANSLATE_FILE_COMMON_LISP);
    { boolean declarationP = declarationTreeP(((Cons*)(statement)));
      Object* translation = incrementallyTranslate(statement);

      if (declarationP &&
          (consP(translation) &&
           (((Cons*)(translation))->value == internCommonLispSymbol("PROGN")))) {
        { Object* elt = NULL;
          int i = NULL_INTEGER;
          Cons* iter000 = ((Cons*)(translation))->rest;
          int iter001 = 1;

          while (!(iter000 == NIL)) {
            elt = iter000->value;
            iter000 = iter000->rest;
            i = iter001;
            iter001 = iter001 + 1;
            if (i == 1) {
              printStellaDefinition(elt, STANDARD_OUTPUT->nativeStream);
            }
            else {
              printStellaCode(elt, STANDARD_OUTPUT->nativeStream);
            }
          }
        }
      }
      else {
        printStellaCode(translation, STANDARD_OUTPUT->nativeStream);
      }
    }
  }
}

boolean auxiliaryVariableUnitP(TranslationUnit* unit) {
  return (unit->auxiliaryP &&
      (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE));
}

void clOutputOneUnit(TranslationUnit* unit, ostream* stream) {
  { Object* translation = unit->translation;

    if (!((boolean)(translation))) {
      cout << "`" << unit << "'" << " has a NULL translation." << endl;
      return;
    }
    if (((boolean)(unit->category))) {
      if (!(auxiliaryVariableUnitP(unit))) {
        *(stream) << endl;
      }
      if (unit->annotation != NULL) {
        *(stream) << ";;; " << unit->annotation << endl << endl;
      }
      if (isaP(translation, SGT_CL_TRANSLATE_FILE_STELLA_CONS) &&
          (((Cons*)(translation))->value == internCommonLispSymbol("PROGN"))) {
        { Object* form = NULL;
          Cons* iter000 = ((Cons*)(translation))->rest;

          while (!(iter000 == NIL)) {
            form = iter000->value;
            iter000 = iter000->rest;
            printStellaDefinition(form, stream);
          }
        }
      }
      else {
        printStellaDefinition(translation, stream);
      }
      unit->category = NULL;
    }
  }
}

void clOutputAllUnitsToFile(char* sourcefile) {
  { char* outputfile = makeFileName(sourcefile, KWD_CL_TRANSLATE_FILE_LISP, FALSE);
    OutputFileStream* outputstream = newOutputFileStream(outputfile);
    ostream* nativeoutputstream = outputstream->nativeStream;

    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Writing " << "`" << outputfile << "'" << "..." << endl;
    }
    outputFileHeader(outputstream, sourcefile);
    *(nativeoutputstream) << ";;; Auxiliary variables:" << endl << endl;
    { TranslationUnit* unit = NULL;
      Cons* iter000 = oTRANSLATIONUNITSo.get()->theConsList;

      while (!(iter000 == NIL)) {
        unit = ((TranslationUnit*)(iter000->value));
        iter000 = iter000->rest;
        if (auxiliaryVariableUnitP(unit)) {
          clOutputOneUnit(unit, nativeoutputstream);
        }
      }
    }
    clOutputForwardDeclarations(nativeoutputstream);
    if (oCURRENT_STELLA_FEATURESo.get()->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS) &&
        (!oCURRENT_STELLA_FEATURESo.get()->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS))) {
      { TranslationUnit* unit = NULL;
        Cons* iter001 = oTRANSLATIONUNITSo.get()->theConsList;

        while (!(iter001 == NIL)) {
          unit = ((TranslationUnit*)(iter001->value));
          iter001 = iter001->rest;
          if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) {
            clOutputOneUnit(unit, nativeoutputstream);
          }
        }
      }
    }
    { TranslationUnit* unit = NULL;
      Cons* iter002 = oTRANSLATIONUNITSo.get()->theConsList;

      while (!(iter002 == NIL)) {
        unit = ((TranslationUnit*)(iter002->value));
        iter002 = iter002->rest;
        if (!auxiliaryVariableUnitP(unit)) {
          clOutputOneUnit(unit, nativeoutputstream);
        }
      }
    }
    outputstream->free();
  }
}

void clOutputFileHeader(OutputStream* stream, char* file) {
  file = file;
  { char* package = oMODULEo.get()->lispPackage();
    boolean ensurepackageP = TRUE;

    *(stream->nativeStream) << ";;; -*- Mode: Lisp; Package: " << package << "; Syntax: COMMON-LISP; Base: 10 -*-" << endl << endl;
    { 
      BIND_STELLA_SPECIAL(oTRANSLATOROUTPUTLANGUAGEo, Keyword*, KWD_CL_TRANSLATE_FILE_COMMON_LISP);
      outputCopyrightHeader(stream);
    }
    if (!(stringEqlP(package, "STELLA") ||
        (!ensurepackageP))) {
      clOutputEnsurePackageDefinition(stream, package);
      *(stream->nativeStream) << endl;
    }
    clOutputInPackageDeclaration(stream, package);
    *(stream->nativeStream) << endl;
  }
}

void clOutputEnsurePackageDefinition(OutputStream* stream, char* package) {
  *(stream->nativeStream) << "(CL:EVAL-WHEN (CL:COMPILE CL:LOAD CL:EVAL)" << endl << "  (CL:UNLESS (CL:FIND-PACKAGE \"" << package << "\")" << endl << "     (CL:DEFPACKAGE \"" << package << "\" (:USE))))" << endl;
}

void clOutputInPackageDeclaration(OutputStream* stream, char* package) {
  *(stream->nativeStream) << "(CL:IN-PACKAGE \"" << package << "\")" << endl;
}

void clOutputForwardDeclarations(ostream* stream) {
  { List* definedvariables = newList();
    List* forwardreferencedvariables = newList();

    { TranslationUnit* unit = NULL;
      Cons* iter000 = oTRANSLATIONUNITSo.get()->theConsList;

      while (!(iter000 == NIL)) {
        unit = ((TranslationUnit*)(iter000->value));
        iter000 = iter000->rest;
        if (auxiliaryVariableUnitP(unit)) {
          definedvariables->push(unit->theObject);
        }
      }
    }
    { TranslationUnit* unit = NULL;
      Cons* iter001 = oTRANSLATIONUNITSo.get()->theConsList;

      while (!(iter001 == NIL)) {
        unit = ((TranslationUnit*)(iter001->value));
        iter001 = iter001->rest;
        if (!auxiliaryVariableUnitP(unit)) {
          if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE) {
            definedvariables->push(unit->theObject);
          }
          { GlobalVariable* variable = NULL;
            Cons* iter002 = unit->referencedGlobals->theConsList;

            while (!(iter002 == NIL)) {
              variable = ((GlobalVariable*)(iter002->value));
              iter002 = iter002->rest;
              if ((!definedvariables->memberP(variable)) &&
                  (!forwardreferencedvariables->memberP(variable))) {
                forwardreferencedvariables->push(variable);
              }
            }
          }
        }
      }
    }
    { ListIterator* it = ((ListIterator*)(forwardreferencedvariables->allocateIterator()));

      while (it->nextP()) {
        it->valueSetter(((GlobalVariable*)(it->value))->variableName);
      }
    }
    if (forwardreferencedvariables->nonEmptyP()) {
      *(stream) << endl << ";;; Forward declarations:" << endl << endl;
      printStellaDefinition(listO(3, internCommonLispSymbol("DECLAIM"), cons(internCommonLispSymbol("SPECIAL"), clTranslateListOfTrees(forwardreferencedvariables->theConsList)->concatenate(NIL, 0)), NIL), stream);
    }
    definedvariables->free();
    forwardreferencedvariables->free();
  }
}

void clTranslateFile(char* file, boolean relativeP) {
  // Translate a Stella `file' to Common-Lisp.  If
  // `relative?', concatenate root directory to `file'.
  if (relativeP) {
    file = makeFileName(file, KWD_CL_TRANSLATE_FILE_STELLA, TRUE);
  }
  translateFile(file, KWD_CL_TRANSLATE_FILE_COMMON_LISP, FALSE);
}

void clTranslateWalkedSystemUnits(KeyValueList* systemunits) {
  { 
    BIND_STELLA_SPECIAL(oCURRENTFILEo, char*, NULL);
    BIND_STELLA_SPECIAL(oTRANSLATIONUNITSo, List*, NULL);
    { Cons* systemfiles = filesPlusSystemStartup(systemDefinitionSourceFiles(oCURRENTSYSTEMDEFINITIONo.get()));
      List* classunits = newList();
      boolean useclstructsP = oCURRENT_STELLA_FEATURESo.get()->membP(KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS);
      char* filename = NULL;

      { StringWrapper* file = NULL;
        List* units = NULL;
        KvCons* iter000 = systemunits->theKvList;

        while (((boolean)(iter000))) {
          file = ((StringWrapper*)(iter000->key));
          units = ((List*)(iter000->value));
          iter000 = iter000->rest;
          oCURRENTFILEo.set(fileBaseName(file->wrapperValue));
          if (useclstructsP) {
            oTRANSLATIONUNITSo.set(newList());
            { TranslationUnit* unit = NULL;
              Cons* iter001 = units->theConsList;

              while (!(iter001 == NIL)) {
                unit = ((TranslationUnit*)(iter001->value));
                iter001 = iter001->rest;
                if (unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) {
                  classunits->push(unit);
                }
                else {
                  oTRANSLATIONUNITSo.get()->push(unit);
                }
              }
            }
            oTRANSLATIONUNITSo.get()->reverse();
          }
          else {
            oTRANSLATIONUNITSo.set(units);
          }
          { 
            BIND_STELLA_SPECIAL(oMODULEo, Module*, computeFileUnitsModule(units));
            BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
            translateAndOutputUnitsToFile(file->wrapperValue, "Common Lisp");
          }
          cleanUpTranslationUnitsSpecial();
        }
      }
      if (useclstructsP) {
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          cout << "Collecting CL structure class definitions..." << endl;
        }
        { StringWrapper* file = NULL;
          Cons* iter002 = filesPlusSystemStartup(systemDefinitionSourceFiles(oCURRENTSYSTEMDEFINITIONo.get()));

          while (!(iter002 == NIL)) {
            file = ((StringWrapper*)(iter002->value));
            iter002 = iter002->rest;
            if (!((boolean)(((List*)(systemunits->lookup(wrapString(filename = makeFileName(file->wrapperValue, KWD_CL_TRANSLATE_FILE_STELLA, TRUE)))))))) {
              oTRANSLATIONUNITSo.set(newList());
              oCURRENTFILEo.set(fileBaseName(filename));
              oTRANSLATIONUNITSo.set(newList());
              operateOnFile(filename, KWD_CL_TRANSLATE_FILE_FINALIZE);
              { TranslationUnit* unit = NULL;
                Cons* iter003 = oTRANSLATIONUNITSo.get()->theConsList;
                Cons* collect000 = NULL;

                while (!(iter003 == NIL)) {
                  unit = ((TranslationUnit*)(iter003->value));
                  iter003 = iter003->rest;
                  if ((unit->category == SYM_CL_TRANSLATE_FILE_STELLA_CLASS) &&
                      (createNativeClassP(((Class*)(unit->theObject))) &&
                       (!((Class*)(unit->theObject))->mixinP))) {
                    if (!((boolean)(collect000))) {
                      {
                        collect000 = cons(unit, NIL);
                        if (classunits->theConsList == NIL) {
                          classunits->theConsList = collect000;
                        }
                        else {
                          addConsToEndOfConsList(classunits->theConsList, collect000);
                        }
                      }
                    }
                    else {
                      {
                        collect000->rest = cons(unit, NIL);
                        collect000 = collect000->rest;
                      }
                    }
                  }
                }
              }
              cleanUpTranslationUnitsSpecial();
            }
          }
        }
        clOutputSystemClStructsFile(classunits);
      }
    }
  }
}

List* sortClStructClasses(List* unsortedclasses) {
  { HashTable* classtable = newHashTable();
    List* roots = list(0);
    List* sortedclasses = list(0);

    { Class* clasS = NULL;
      Cons* iter000 = unsortedclasses->theConsList;

      while (!(iter000 == NIL)) {
        clasS = ((Class*)(iter000->value));
        iter000 = iter000->rest;
        classtable->insertAt(clasS->classType, TRUE_WRAPPER);
      }
    }
    { Class* clasS = NULL;
      Cons* iter001 = unsortedclasses->theConsList;
      Cons* collect000 = NULL;

      while (!(iter001 == NIL)) {
        clasS = ((Class*)(iter001->value));
        iter001 = iter001->rest;
        { boolean testValue000 = FALSE;

          { boolean foundP000 = FALSE;

            { Surrogate* super = NULL;
              Cons* iter002 = clasS->classDirectSupers->theConsList;

              while (!(iter002 == NIL)) {
                super = ((Surrogate*)(iter002->value));
                iter002 = iter002->rest;
                if (((boolean)(((BooleanWrapper*)(classtable->lookup(super)))))) {
                  foundP000 = TRUE;
                  break;
                }
              }
            }
            testValue000 = foundP000;
          }
          testValue000 = !testValue000;
          if (testValue000) {
            if (!((boolean)(collect000))) {
              {
                collect000 = cons(clasS, NIL);
                if (roots->theConsList == NIL) {
                  roots->theConsList = collect000;
                }
                else {
                  addConsToEndOfConsList(roots->theConsList, collect000);
                }
              }
            }
            else {
              {
                collect000->rest = cons(clasS, NIL);
                collect000 = collect000->rest;
              }
            }
          }
        }
      }
    }
    while (roots->nonEmptyP()) {
      sortedclasses->push(((Class*)(roots->first())));
      { Surrogate* sub = NULL;
        Cons* iter003 = ((Class*)(roots->pop()))->classDirectSubs->theConsList;

        while (!(iter003 == NIL)) {
          sub = ((Surrogate*)(iter003->value));
          iter003 = iter003->rest;
          if (((boolean)(((BooleanWrapper*)(classtable->lookup(sub)))))) {
            roots->push(((Class*)(sub->surrogateValue)));
          }
        }
      }
    }
    return (sortedclasses->reverse());
  }
}

char* clYieldStructClassFileName(char* systemname) {
  return (stringConcatenate(stringDowncase(systemname), "-system-structs", 0));
}

void clOutputSystemClStructsFile(List* classunits) {
  { char* outputfile = makeFileName(clYieldStructClassFileName(oCURRENTSYSTEMDEFINITIONo.get()->name), KWD_CL_TRANSLATE_FILE_LISP, TRUE);
    OutputFileStream* outputstream = newOutputFileStream(outputfile);
    List* classes = newList();
    Module* currentmodule = oMODULEo.get();
    char* package = NULL;
    List* ensuredpackages = list(0);

    { TranslationUnit* unit = NULL;
      Cons* iter000 = classunits->theConsList;
      Cons* collect000 = NULL;

      while (!(iter000 == NIL)) {
        unit = ((TranslationUnit*)(iter000->value));
        iter000 = iter000->rest;
        if (!((boolean)(collect000))) {
          {
            collect000 = cons(unit->theObject, NIL);
            if (classes->theConsList == NIL) {
              classes->theConsList = collect000;
            }
            else {
              addConsToEndOfConsList(classes->theConsList, collect000);
            }
          }
        }
        else {
          {
            collect000->rest = cons(unit->theObject, NIL);
            collect000 = collect000->rest;
          }
        }
      }
    }
    classes = sortClStructClasses(classes);
    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Writing " << "`" << outputfile << "'" << "..." << endl;
    }
    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, oMODULEo.get());
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
      if (classes->nonEmptyP()) {
        oMODULEo.set(((Class*)(classes->first()))->homeModule());
      }
      else {
        oMODULEo.set(getStellaModule(oCURRENTSYSTEMDEFINITIONo.get()->cardinalModule, TRUE));
      }
      currentmodule = oMODULEo.get();
      clOutputFileHeader(outputstream, NULL);
      { TranslationUnit* unit = NULL;
        Class* clasS = NULL;
        Cons* iter001 = classunits->theConsList;
        Cons* iter002 = classes->theConsList;

        while ((!(iter001 == NIL)) &&
            (!(iter002 == NIL))) {
          unit = ((TranslationUnit*)(iter001->value));
          iter001 = iter001->rest;
          clasS = ((Class*)(iter002->value));
          iter002 = iter002->rest;
          unit->theObject = clasS;
          unit->annotation = NULL;
          oMODULEo.set(clasS->homeModule());
          if (!(oMODULEo.get() == currentmodule)) {
            currentmodule = oMODULEo.get();
            package = oMODULEo.get()->lispPackage();
            if (!(ensuredpackages->memberP(wrapString(package)))) {
              ensuredpackages->push(wrapString(package));
              *(outputstream->nativeStream) << endl;
              clOutputEnsurePackageDefinition(outputstream, package);
            }
            *(outputstream->nativeStream) << endl;
            clOutputInPackageDeclaration(outputstream, package);
          }
          unit->translation = clTranslateDefineNativeClassUnit(unit);
          clOutputOneUnit(unit, outputstream->nativeStream);
        }
      }
    }
    outputstream->free();
  }
}

void clTranslateSystem(char* systemName) {
  // Translate a Stella system named `system-name' to Common Lisp.
  translateSystem(systemName, KWD_CL_TRANSLATE_FILE_COMMON_LISP, 6, KWD_CL_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, TRUE_WRAPPER);
}

void clCompileAndLoadFile(char* file, boolean relativeP, boolean forcerecompilationP) {
  { char* translatedfile = makeFileName(file, KWD_CL_TRANSLATE_FILE_LISP, relativeP);
    char* binaryfile = makeFileName(file, KWD_CL_TRANSLATE_FILE_LISP_BINARY, relativeP);

  }
}

void clTranslateStella(boolean productionsettingsP) {
  translateSystem("stella", KWD_CL_TRANSLATE_FILE_COMMON_LISP, 4, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, (productionsettingsP ? TRUE_WRAPPER : FALSE_WRAPPER));
}

void clRetranslateStella(boolean productionsettingsP) {
  translateSystem("stella", KWD_CL_TRANSLATE_FILE_COMMON_LISP, 6, KWD_CL_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp, (productionsettingsP ? TRUE_WRAPPER : FALSE_WRAPPER));
}

void startupClTranslateFile() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (currentStartupTimePhaseP(2)) {
      KWD_CL_TRANSLATE_FILE_COMMON_LISP = ((Keyword*)(internRigidSymbolWrtModule("COMMON-LISP", NULL, 2)));
      SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(internRigidSymbolWrtModule("GLOBAL-VARIABLE", NULL, 0)));
      SGT_CL_TRANSLATE_FILE_STELLA_CONS = ((Surrogate*)(internRigidSymbolWrtModule("CONS", NULL, 1)));
      KWD_CL_TRANSLATE_FILE_LISP = ((Keyword*)(internRigidSymbolWrtModule("LISP", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS = ((Keyword*)(internRigidSymbolWrtModule("USE-COMMON-LISP-STRUCTS", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS = ((Keyword*)(internRigidSymbolWrtModule("USE-COMMON-LISP-VECTOR-STRUCTS", NULL, 2)));
      SYM_CL_TRANSLATE_FILE_STELLA_CLASS = ((Symbol*)(internRigidSymbolWrtModule("CLASS", NULL, 0)));
      KWD_CL_TRANSLATE_FILE_STELLA = ((Keyword*)(internRigidSymbolWrtModule("STELLA", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_FINALIZE = ((Keyword*)(internRigidSymbolWrtModule("FINALIZE", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_TWO_PASSp = ((Keyword*)(internRigidSymbolWrtModule("TWO-PASS?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp = ((Keyword*)(internRigidSymbolWrtModule("FORCE-TRANSLATION?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp = ((Keyword*)(internRigidSymbolWrtModule("PRODUCTION-SETTINGS?", NULL, 2)));
      KWD_CL_TRANSLATE_FILE_LISP_BINARY = ((Keyword*)(internRigidSymbolWrtModule("LISP-BINARY", NULL, 2)));
      SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE = ((Symbol*)(internRigidSymbolWrtModule("STARTUP-CL-TRANSLATE-FILE", NULL, 0)));
      SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(internRigidSymbolWrtModule("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (currentStartupTimePhaseP(6)) {
      finalizeClasses();
    }
    if (currentStartupTimePhaseP(7)) {
      defineFunctionObject("LPTRANS", "(DEFUN LPTRANS ((STATEMENT OBJECT)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Translate `statement' to Common-Lisp and print the result.\")", ((cpp_function_code)(&lptrans)), NULL);
      defineFunctionObject("PTRANS", "(DEFUN PTRANS ((STATEMENT OBJECT)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Translate `statement' to Common-Lisp and print the result.\")", ((cpp_function_code)(&ptrans)), NULL);
      defineFunctionObject("HELP-LPTRANS", "(DEFUN HELP-LPTRANS ((STATEMENT OBJECT)))", ((cpp_function_code)(&helpLptrans)), NULL);
      defineFunctionObject("AUXILIARY-VARIABLE-UNIT?", "(DEFUN (AUXILIARY-VARIABLE-UNIT? BOOLEAN) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&auxiliaryVariableUnitP)), NULL);
      defineFunctionObject("CL-OUTPUT-ONE-UNIT", "(DEFUN CL-OUTPUT-ONE-UNIT ((UNIT TRANSLATION-UNIT) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&clOutputOneUnit)), NULL);
      defineFunctionObject("CL-OUTPUT-ALL-UNITS-TO-FILE", "(DEFUN CL-OUTPUT-ALL-UNITS-TO-FILE ((SOURCEFILE FILE-NAME)))", ((cpp_function_code)(&clOutputAllUnitsToFile)), NULL);
      defineFunctionObject("CL-OUTPUT-FILE-HEADER", "(DEFUN CL-OUTPUT-FILE-HEADER ((STREAM OUTPUT-STREAM) (FILE FILE-NAME)))", ((cpp_function_code)(&clOutputFileHeader)), NULL);
      defineFunctionObject("CL-OUTPUT-ENSURE-PACKAGE-DEFINITION", "(DEFUN CL-OUTPUT-ENSURE-PACKAGE-DEFINITION ((STREAM OUTPUT-STREAM) (PACKAGE STRING)))", ((cpp_function_code)(&clOutputEnsurePackageDefinition)), NULL);
      defineFunctionObject("CL-OUTPUT-IN-PACKAGE-DECLARATION", "(DEFUN CL-OUTPUT-IN-PACKAGE-DECLARATION ((STREAM OUTPUT-STREAM) (PACKAGE STRING)))", ((cpp_function_code)(&clOutputInPackageDeclaration)), NULL);
      defineFunctionObject("CL-OUTPUT-FORWARD-DECLARATIONS", "(DEFUN CL-OUTPUT-FORWARD-DECLARATIONS ((STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&clOutputForwardDeclarations)), NULL);
      defineFunctionObject("CL-TRANSLATE-FILE", "(DEFUN CL-TRANSLATE-FILE ((FILE FILE-NAME) (RELATIVE? BOOLEAN)) :DOCUMENTATION \"Translate a Stella `file' to Common-Lisp.  If\n`relative?', concatenate root directory to `file'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&clTranslateFile)), NULL);
      defineFunctionObject("CL-TRANSLATE-WALKED-SYSTEM-UNITS", "(DEFUN CL-TRANSLATE-WALKED-SYSTEM-UNITS ((SYSTEMUNITS SYSTEM-UNITS-ALIST)))", ((cpp_function_code)(&clTranslateWalkedSystemUnits)), NULL);
      defineFunctionObject("SORT-CL-STRUCT-CLASSES", "(DEFUN (SORT-CL-STRUCT-CLASSES (LIST OF CLASS)) ((UNSORTEDCLASSES (LIST OF CLASS))))", ((cpp_function_code)(&sortClStructClasses)), NULL);
      defineFunctionObject("CL-YIELD-STRUCT-CLASS-FILE-NAME", "(DEFUN (CL-YIELD-STRUCT-CLASS-FILE-NAME FILE-NAME) ((SYSTEMNAME STRING)))", ((cpp_function_code)(&clYieldStructClassFileName)), NULL);
      defineFunctionObject("CL-OUTPUT-SYSTEM-CL-STRUCTS-FILE", "(DEFUN CL-OUTPUT-SYSTEM-CL-STRUCTS-FILE ((CLASSUNITS (LIST OF TRANSLATION-UNIT))))", ((cpp_function_code)(&clOutputSystemClStructsFile)), NULL);
      defineFunctionObject("CL-TRANSLATE-SYSTEM", "(DEFUN CL-TRANSLATE-SYSTEM ((SYSTEM-NAME STRING)) :DOCUMENTATION \"Translate a Stella system named `system-name' to Common Lisp.\" :PUBLIC? TRUE)", ((cpp_function_code)(&clTranslateSystem)), NULL);
      defineFunctionObject("CL-COMPILE-AND-LOAD-FILE", "(DEFUN CL-COMPILE-AND-LOAD-FILE ((FILE STRING) (RELATIVE? BOOLEAN) (FORCERECOMPILATION? BOOLEAN)))", ((cpp_function_code)(&clCompileAndLoadFile)), NULL);
      defineFunctionObject("CL-TRANSLATE-STELLA", "(DEFUN CL-TRANSLATE-STELLA ((PRODUCTIONSETTINGS? BOOLEAN)))", ((cpp_function_code)(&clTranslateStella)), NULL);
      defineFunctionObject("CL-RETRANSLATE-STELLA", "(DEFUN CL-RETRANSLATE-STELLA ((PRODUCTIONSETTINGS? BOOLEAN)))", ((cpp_function_code)(&clRetranslateStella)), NULL);
      defineFunctionObject("STARTUP-CL-TRANSLATE-FILE", "(DEFUN STARTUP-CL-TRANSLATE-FILE () :PUBLIC? TRUE)", ((cpp_function_code)(&startupClTranslateFile)), NULL);
      { MethodSlot* function = lookupFunction(SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE);

        setDynamicSlotValue(function->dynamicSlots, SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, wrapString("Startup-Cl-Translate-File"), NULL_STRING_WRAPPER);
      }
    }
    if (currentStartupTimePhaseP(8)) {
      finalizeSlots();
      cleanupUnfinalizedClasses();
    }
  }
}

Keyword* KWD_CL_TRANSLATE_FILE_COMMON_LISP = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = NULL;

Surrogate* SGT_CL_TRANSLATE_FILE_STELLA_CONS = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_LISP = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_STRUCTS = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_USE_COMMON_LISP_VECTOR_STRUCTS = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_CLASS = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_STELLA = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_FINALIZE = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_TWO_PASSp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_FORCE_TRANSLATIONp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_PRODUCTION_SETTINGSp = NULL;

Keyword* KWD_CL_TRANSLATE_FILE_LISP_BINARY = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_STARTUP_CL_TRANSLATE_FILE = NULL;

Symbol* SYM_CL_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
